#!/bin/sh
#
#	@(#)edit_config.sh	1.3 93/01/15 Copyright (C) 1992  Sun Microsystems, Inc

Error () {
	fmt -79 <<- EOF
	
	$*
	
	EOF
	exit 1
}

TMP1=/tmp/snc.1.$$
TMP2=/tmp/snc.2.$$
PATTERNS_SED=/tmp/patterns.sed.$$
DIR=${PROTO}/sys/${KARCH:=`arch -k`}/conf
unset ORIG

MAXUSERS=32	# new minimum for "maxusers" number in config file

###################################################################
# Shell function to get a piece of a word.
# Customer machine may not have /usr/bin/cut, so we use awk
# if cut is missing.
###################################################################

SubString () {
  if [ -x /usr/bin/cut ] ; then
    echo $1 | cut -c$2-
  else
    echo $1 $2 | awk '{print substr($1,$2,length($1)-$2+1)}' -
  fi
}

###################################################################
# Shell function to run a sub-shell.
# First argument starts with a !, so we cut it
###################################################################

SubShell () {
  echo ''
  case $# in
    0)
	echo 'Type "exit" to return to the installation process.'
    	${SHELL:=/bin/sh}
    	;;
    1)
    	case $1 in
    	  !)	echo 'Type "exit" to return to the installation process.'
		${SHELL:=/bin/sh}
		;;
    	  !*)	cmd=`SubString $1 2` ; $cmd  ;;
    	  *)	echo 'Error in SubShell()' ; exit 255 ;;
    	esac
    	;;
    *)
	case $1 in
	  !)	cmd=$2 ; shift 2 ;;
	  !*)	cmd=`SubString $1 2` ; shift ;;
	  *)	echo 'Error in SubShell()' ; exit 255 ;;
	esac
	$cmd $*
	;;
  esac
  echo ''
  return 0
}

###################################################################
# Shell function to get a random response (into variable $ANS)
###################################################################

Response () {
  while true; do
    echo -n "
$* "
    read ANS
    case "$ANS" in
      !*)	SubShell $ANS ;;
      *)	return 0 ;;
    esac
  done
}

###################################################################
# Try various methods of discerning the current kernel name.
###################################################################

# Note: following code is SunOS dependent.  We try real hard to find
# a kernel name in the motd file, the dmesg output, and in the kernel
# binary itself.  If all fails, we ask the user.

# A sed expression to get the kernel identifier out of the herald string:
sed_exp='s/^SunOS Release.* (\(.*\)) .*$/\1/p'

if [ -f $TOP_DIR/etc/motd ] ; then
	ORIG=`sed -n -e "$sed_exp" $TOP_DIR/etc/motd`
	FULLNAME="$TOP_DIR$DIR/$ORIG"
	if [ ! -f $FULLNAME ]
	then
		ORIG=""
	fi
fi
if [ -f $TOP_DIR/vmunix -a ! "$ORIG" ] ; then
	ORIG=`strings $TOP_DIR/vmunix | sed -n -e "$sed_exp"`
	FULLNAME="$TOP_DIR$DIR/$ORIG"
	if [ ! -f $FULLNAME ]
	then
		ORIG=""
	fi
fi
if [ ! "$ORIG" ] ; then
	ORIG=`dmesg | grep '^SunOS' | sed -n -e "\$ $sed_exp"`
	FULLNAME="$TOP_DIR$DIR/$ORIG"
	if [ ! -f $FULLNAME ]
	then
		ORIG=""
	fi
fi
if [ -f $TOP_DIR/vmunix -a ! "$ORIG" ] ; then
	ORIG=`nm -a $TOP_DIR/vmunix | grep "/usr/kvm/sys/$KARCH" | \
		sed -e "s~^.*/usr/kvm/sys/$KARCH/\(.*\)/.*$~\1~"`
	FULLNAME="$TOP_DIR$DIR/$ORIG"
	if [ ! -f $FULLNAME ]
	then
		ORIG=""
	fi
fi
if [ -f /usr/adm/messages -a ! "$ORIG" ]
then
	ORIG=`egrep SunOS /usr/adm/messages | tail -1 | \
	    sed -e 's/.*(\(.*\)).*/\1/'`
	FULLNAME="$TOP_DIR$DIR/$ORIG"
	if [ ! -f $FULLNAME ]
	then
		ORIG=""
	fi
fi

if [ ! "$ORIG" ]
then
	echo 'Could not determine current kernel name.'
	ok_to_punt=true
	while [ ! "$ORIG" ] ; do
	  Response Please type the kernel config file name:
	  if $ok_to_punt && [ ! "$ANS" ] ; then
	    break
	  fi
	  if [ -f "$TOP_DIR$DIR/$ANS" ] ; then
	    ORIG="$ANS"
	  elif [ -f $TOP_DIR$DIR/GENERIC ] ; then
	    more <<- EOF
		File "$TOP_DIR$DIR/$ANS" does not exist.
		
		If you do not know the name of the kernel, try "GENERIC".
		While this will result in a kernel which is too large,
		it will at least get you started.
		
		EOF
	  else
	    more <<- EOF
		File "$TOP_DIR$DIR/$ANS" does not exist.
		
		If you do not know the name of the kernel, type a "!" at the
		next prompt to get a sub-shell and look around your
		$TOP_DIR$DIR/conf directory for possible kernel names.
		Then type "exit" to continue with this installation.
		
		If all else fails, just hit RETURN and we will skip this step.
		
		EOF
	  fi
	done
fi

[ "$ORIG" ] || exit 1

# Make copy with "_SNC" appended if "_SNC" not already part of the name
res=`expr match ${ORIG} ".*_SNC.*"`
if [ $res -eq 0 ]
then
	FILE=${ORIG}_SNC
else
	FILE=${ORIG}
fi
FULLNAME="$TOP_DIR$DIR/$FILE"

# Save the SNC kernel name in a file for use by the main install script.
if [ ! -d /usr/tmp/unbundled/SNC ]
then
	mkdir -p /usr/tmp/unbundled/SNC
fi

cd $TOP_DIR$DIR

###################################################################
# Let user override the name for the kernel
# Check that file exists and has not been SNC-ified.
###################################################################

echo
echo Current kernel is named \"$ORIG\".
echo Going to use kernel name \"$FILE\".
Response "Would you like to change the name [yn]?"
if [ "$ANS" = "y" ]
then
	FILE=
	while [ ! "$FILE" ] ; do
	  Response Please type the kernel config file name:
	  if $ok_to_punt && [ ! "$ANS" ] ; then
	    break
	  fi
	  if [ -f "$TOP_DIR$DIR/$ANS" ] ; then
	    FILE="$ANS"
	  elif [ -f $TOP_DIR$DIR/GENERIC ] ; then
	    more <<- EOF
		File "$TOP_DIR$DIR/$ANS" does not exist.
		
		If you do not know the name of the kernel, try "GENERIC".
		While this will result in a kernel which is too large,
		it will at least get you started.
		
		EOF
	  else
	    more <<- EOF
		File "$TOP_DIR$DIR/$ANS" does not exist.
		
		If you do not know the name of the kernel, type a "!" at the
		next prompt to get a sub-shell and look around your
		$TOP_DIR$DIR/conf directory for possible kernel names.
		Then type "exit" to continue with this installation.
		
		If all else fails, just hit RETURN and we will skip this step.
		
		EOF
	  fi
	done
fi

if [ -f $FILE ] && grep -sw SNCNET $FILE
then
	echo $FILE has already been modified for $PRODUCT.
	exit 0
fi
if [ ! -f $ORIG ] ; then
	Error Could not find config file $TOP_DIR$DIR/$ORIG -- \
		make sure this machine is equipped to build kernels.
fi
if grep -sw SNCNET $ORIG
then
	echo $ORIG has already been modified for $PRODUCT.
	echo New kernel will be named \"$FILE\".
	cp $ORIG $FILE
	exit 0
fi

echo $FILE > /usr/tmp/unbundled/SNC/kernelName.ne

###################################################################
# Check that original file contains the key strings.
###################################################################

if grep -s "^maxusers${WS}[0-9][0-9]*.*$" $ORIG &&
   grep -s "^options\>" $ORIG &&
   grep -s "^ident\>" $ORIG
then
	: OK
else
	Error Cannot edit $FULLNAME -- a required pattern \
	'(either' '"^maxusers"' or '"^options"' or '"^ident")' \
	is missing. \
	This probably indicates that $FULLNAME has been edited beyond \
	the recognition of this script.  Please see the SNC Installation \
	Manual for instructions on manual editing.
fi

###################################################################
# Announce the new kernel name.
###################################################################

echo
echo New kernel will be named \"$FILE\".
echo

# max 8 for Sun-4/600MP
if [ $KARCH = sun4m ]
then
	MAXNCS=8
else
	MAXNCS=6
fi

if [ "$ORIG" = "GENERIC" ] ; then
	# In a generic kernel installation, we pretend the maximum
	# number of SNCs is installed.
	fmt -79 <<- EOF

	NOTE: performance considerations dictate that you NOT use a
	generic kernel configuration.  Please consult your $OS_VER
	"System and Network Administration" manual, chapter 9,
	"Reconfiguring the System Kernel" for more information.  Then
	you will be able to reconfigure your kernel for best
	performance.

	EOF
fi

###################################################################
# Determine which "device ne*" lines to enable and disable.
###################################################################

# This next bit of shell abuse gets the line numbers of the "device ne*"
# lines and sets up some sed patterns in order to ensure that the right
# number of ne devices are enabled in the config file.  This assumes the
# ne numbers are contiguous.

{
  NC=0
  while [ $NC -lt $MAXNCS ] ; do
    line1=`grep -n "^#*device${WS}ne${NC}${WS}at" ${INSTALL_DIR}/SNC/${KARCH}/config.add.nc.b | 
	   sed -e "s/^\([^:]*\):.*$/\1/"`
    line2=`expr $line1 + 1`
    if [ $NC -lt $MAXNCS ] ; then	# if in range, then
      echo "$line1,$line2 s/^##*//"	# un-comment these lines
    else				# else
      echo "$line1,$line2 s/^[^#]/#&/"	# comment out these lines
    fi
    NC=`expr $NC + 1`
  done
} > PATTERNS_SED

###################################################################
# Perform the edit.
###################################################################

echo Editing $FULLNAME

# First do the ident edit.  Rather than simply changing strings in
# the config file, we comment out the old ones and add the new ones
# directly below.
sed -e "
/^ident${WS}/ {
s/ident/#ident/
a\\
ident		\"$FILE\"
}
" $ORIG > $TMP1

# Do the maxusers edit only if needed, i.e., if the existing
# value of maxusers is less than $MAXUSERS.
maxusers=`sed -n -e "s/^maxusers${WS}\([0-9][0-9]*\).*$/\1/p" $ORIG`
if [ $maxusers -lt $MAXUSERS ] ; then
	mv $TMP1 $TMP2
	sed -e "
/^maxusers${WS}[0-9][0-9]*/ {
s/maxusers/#maxusers/
a\\
maxusers	$MAXUSERS	# increased for Sun Network CoProcessor
}
" $TMP2 > $TMP1
fi

# Now locate the last existing options line.
optLast=`grep -n '^options\>' $TMP1 | tail -1 | awk -F: '{print $1}'`
optRest=`expr $optLast + 1`

# Finally concatenate the pieces together.
head -$optLast $TMP1 > $FILE
sed -e "/^###${WS}@(#)/p" -e '/^###/d' ${INSTALL_DIR}/SNC/${KARCH}/config.add.nc.a >> $FILE
tail +$optRest $TMP1 >> $FILE
sed -e "/^###${WS}@(#)/p" -e '/^###/d' -f PATTERNS_SED ${INSTALL_DIR}/SNC/${KARCH}/config.add.nc.b >> $FILE

rm -f $TMP1 $TMP2 PATTERNS_SED
NEWCONFIGFILE=FILE
export NEWCONFIGFILE
exit 0
